@@ -2,9 +2,10 @@ use rust_embed::EmbeddedFile;
 use serde::Deserialize;
 
 use std::collections::HashMap;
-use std::error::Error;
 use std::str;
 
+use crate::error::Error;
+
 #[derive(Debug, Deserialize)]
 pub struct Pokemon {
     pub idx: u32,
@@ -15,8 +16,8 @@ pub struct Pokemon {
     pub desc: HashMap<String, String>,
 }
 
-pub fn load_pokemon(pokemon_db: &EmbeddedFile) -> Result<Vec<Pokemon>, Box<dyn Error>> {
-    let pokemon_json_str = str::from_utf8(&pokemon_db.data)?;
+pub fn load_pokemon(pokemon_db: &EmbeddedFile) -> Result<Vec<Pokemon>, Error> {
+    let pokemon_json_str = str::from_utf8(&pokemon_db.data).expect("Invalid UTF-8 in pokemon db");
     let pokemon: Vec<Pokemon> = serde_json::from_str(pokemon_json_str)?;
     Ok(pokemon)
 }
